package worldclient.app;

import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;
import org.jvnet.substance.skin.SubstanceRavenGraphiteLookAndFeel;
import worldclient.controller.WorldClientController;
import worldclient.model.WorldClientModel;
import worldclient.model.WorldClientState;
import worldclient.view.WorldClientView;

/**
 *
 * @author Fernando Martines (RA: 087570)
 */
public class WorldClientApp {

    public static final boolean VERBOSE_LOG = true;
    
    private static WorldClientApp instance;
    
    private WorldServerConnection worldServerConnection;
    
    private WorldClientController worldClientController;
    private WorldClientView worldClientView;
    
    private WorldClientState worldClientState;
    private WorldClientModel worldClientModel;

    private WorldClientApp() {        
    }
    
    public static WorldClientApp getInstance() {
        if (instance == null)
            instance = new WorldClientApp();
        return instance;
    }
    
    public void launch() {

        // create application state
        worldClientState = new WorldClientState();
        
        // start main view
        SwingUtilities.invokeLater(new Runnable() {
            @Override
            public void run() {
                try {
                    // Define o look and feel.
                    UIManager.setLookAndFeel(new SubstanceRavenGraphiteLookAndFeel());
                } catch (UnsupportedLookAndFeelException ex) {
                    System.err.println("No foi possvel carregar Substance Look and Feel.");
                    System.err.println("Carregando Look and Feel padro...");
                }
                
                startApplication();
            }
        });
    }
    
    public void startApplication() {
        worldClientModel = new WorldClientModel();

        worldClientView = new WorldClientView();
        worldClientView.setVisible(true);

        worldClientController = new WorldClientController();
        worldClientController.startObservers();

        showStatus("Connecting...");
        SwingUtilities.invokeLater(new Runnable() {
            @Override
            public void run() {
                worldServerConnection = new WorldServerConnection();
                worldServerConnection.connect();
            }
        });
    }

    public void showInfoMessage(String message) {
        worldClientView.getLabelStatus().setText("<html><font color=\"#80a0ff\">" + message);
    }

    public void showErrorMessage(String message) {
        worldClientView.getLabelStatus().setText("<html><font color=\"#ffa080\">" + message);
    }
    
    public void showStatus(String message) {
        worldClientView.getLabelStatus().setText("<html><font color=\"white\">" + message);
    }
    public WorldClientController getWorldClientController() {
        return worldClientController;
    }

    public WorldClientView getWorldClientView() {
        return worldClientView;
    }

    public WorldClientState getState() {
        return worldClientState;
    }

    public void setState(WorldClientState worldClientState) {
        this.worldClientState = worldClientState;
    }


    public WorldServerConnection getWorldServerConnection() {
        return worldServerConnection;
    }

    public void setWorldServerConnection(WorldServerConnection worldServerConnection) {
        this.worldServerConnection = worldServerConnection;
    }

    public WorldClientModel getModel() {
        return worldClientModel;
    }

    public void setModel(WorldClientModel worldClientModel) {
        this.worldClientModel = worldClientModel;
    }
    
}
