package worldclient.app;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.net.Socket;
import java.net.UnknownHostException;
import java.util.logging.Level;
import java.util.logging.Logger;
import worldclient.observer.WorldServerObserver;
import worldclient.util.Constants;

/**
 *
 * @author Fernando.Martines
 */
public class WorldServerConnection {
    
    private Socket socket;    
    private ServerThread serverThread;
    
    private PrintWriter out;
    private BufferedReader in;
    
    private WorldClientApp app = WorldClientApp.getInstance();

    public WorldServerConnection() {
    }
    
    public boolean connect() {
        return this.connect(Constants.WORLD_SERVER_DEFAULT_IP_ADDRESS, Constants.WORLD_SERVER_DEFAULT_PORT);
    }

    public boolean connect(String addr, int port) {
        
        String message = "";
        try {
           socket = new Socket(
                    Constants.WORLD_SERVER_DEFAULT_IP_ADDRESS, 
                    Constants.WORLD_SERVER_DEFAULT_PORT);
        } catch (UnknownHostException ex) {
            message = ex.getMessage();
            Logger.getLogger(WorldServerObserver.class.getName()).log(Level.SEVERE, null, "[WorldServer Connection] Host unknown: " + Constants.WORLD_SERVER_DEFAULT_IP_ADDRESS);
            
        } catch (IOException ex) {
            message = ex.getMessage();
            Logger.getLogger(WorldServerObserver.class.getName()).log(Level.SEVERE, null, "[WorldServer Connection] Can't connect to WorldServer " + Constants.WORLD_SERVER_DEFAULT_IP_ADDRESS + ":" + Constants.WORLD_SERVER_DEFAULT_PORT);
        }
        
        boolean connected = socket != null && socket.isConnected();
        
        WorldClientApp.getInstance().getModel().getWorld().setConnected(connected);

        if (connected) {
            testConnection();
        }
        
        return connected;
    }
    
    public void testConnection() {
        String inputLine, outputLine;
        boolean ok = false;
        try {
            out = new PrintWriter(socket.getOutputStream(), true);
            in = new BufferedReader(new InputStreamReader(socket.getInputStream()));
            ok = true;
        } catch (Throwable t) {
            System.out.println("Caught " + t + " - closing thread");
        }
        
        if (!ok) {
            app.showErrorMessage("Invalid Input/Output Streams.");
            app.getModel().getWorld().setStarted(false);
        }

        if (serverThread != null)
            serverThread.interrupt();
        
        serverThread = new ServerThread(socket);
    }

    public void sendMessage(String message) {
        if (WorldClientApp.VERBOSE_LOG) {
            System.out.println(">>> " + message);
            System.out.flush();
        }
        
        out.println(message);
        out.flush();
    }

    public Socket getSocket() {
        return socket;
    }

    private class ServerThread extends Thread {

        private final Socket socket;
        private String name;

        public ServerThread(Socket socket) {
            System.out.println("### WorldServer ###");
            this.socket = socket;
            start();
        }

        public void run() {
            int cond = 0;
            String inputLine;
            try {
                sendMessage(WorldServerProtocol.VERSION_MESSAGE);
                while (cond == 0) {
                    inputLine = in.readLine();
                    processMessage(inputLine);
                    if (inputLine.equals("shutdown")) {
                        System.exit(0);
                    }
                    if (inputLine.equals("quit")) {
                        cond = 1;
                    }
                }
                socket.close();
            } catch (Throwable t) {
                System.out.println("Caught " + t + " - closing thread");
            }
        }
    }

    public synchronized String processMessage(String inputMessage) {
        if (WorldClientApp.VERBOSE_LOG)
            System.out.println("<<< " + inputMessage);

        WorldServerProtocol.parseMessage(inputMessage);
        
        return inputMessage;
    }

}
