package worldclient.app;

import java.awt.Point;
import java.text.DecimalFormat;

/**
 *
 * @author Fernando.Martines
 */
public class WorldServerProtocol {

    public static final String VERSION_MESSAGE = "version";
    public static final String NEW_CREATURE_MESSAGE = "new _x_ _y_ _p_";
    public static final String GET_ENVIRONMENT_MESSAGE = "getenvironment";
    public static final String CREATURE_COORDS_MESSAGE = "getcreatcoords _id_";
    public static final String CREATURE_INFO_MESSAGE = "getcreatinfo _id_";
    public static final String SET_ANGLE_MESSAGE = "setAngle _id_ _v_ _v_ _a_";
    public static final String START_CREATURE_MESSAGE = "start _id_";
    public static final String STOP_CREATURE_MESSAGE = "stop _id_";
    public static final String CAMERA_MESSAGE = "camera _id_";
    public static final String REFUEL_MESSAGE = "refuel _id_";

    private static WorldClientApp app = WorldClientApp.getInstance();
    
    public static void parseMessage(String inputMessage) {
        if (inputMessage.toLowerCase().contains("welcome"))
            app.getModel().getWorld().setStarted(true);
        else if (inputMessage.toLowerCase().contains("environment")) 
            parseEnvironmentMessage(inputMessage);
        else if (inputMessage.toLowerCase().contains("new creature")) 
            parseNewCreateMessage(inputMessage);
        else if (inputMessage.toLowerCase().contains("camera attached")) 
            parseCameraAttachedMessage(inputMessage);
        else if (inputMessage.toLowerCase().contains("fuel")) 
            parseCreatureInfoFuelLevelMessage(inputMessage);
        else if (inputMessage.toLowerCase().contains("creature run")) 
            parseCreatureStartedMessage(inputMessage);
        else if (inputMessage.toLowerCase().contains("creature has stopped")) 
            parseCreatureStoppedMessage(inputMessage);
        else if (inputMessage.toLowerCase().contains("creature coordinates")) 
            parseCreatureCoordsMessage(inputMessage);
    }
    
    private static void parseEnvironmentMessage(String message) {
        // "Environment:800 600"
        try {
            String[] sDimension = message.split(":");
            sDimension = sDimension[1].trim().split(" ");
            String sWidth = sDimension[0].trim();
            String sHeight = sDimension[1].trim();
            int width = Integer.parseInt(sWidth);
            int height = Integer.parseInt(sHeight);
            Point point = new Point(width, height);
            app.getModel().getWorld().setEnvironmentDimension(point);
            app.getModel().getWorld().setRunning(true);
        } catch (Exception e) {
            app.getModel().getWorld().setStarted(false);
            app.getModel().getWorld().setRunning(false);
            app.getModel().getWorld().setEnvironmentDimension(new Point(0, 0));
            System.err.println("PARSER ERROR: " + e.getMessage());
        }
    }

    private static void parseNewCreateMessage(String message) {
        // "New Creature created: x: 400.0 y: 300.0 pitch: 0.0"
        int id = app.getModel().getCreature().getId();
        app.getModel().getCreature().setId(id + 1);
        app.getModel().getCreature().notifyObservers();
    }
    
    private static void parseCameraAttachedMessage(String message) {
        // "Even Camera attached to robot 0"
        try {
            String[] tokens = message.split(" ");
            String sCameraId = tokens[tokens.length - 1] ;
            int cameraId = Integer.parseInt(sCameraId);
            app.getModel().getCreature().setCamera(cameraId);
        } catch (Exception e) {
            app.getModel().getWorld().setStarted(false);
            app.getModel().getWorld().setRunning(false);
            app.getModel().getWorld().setEnvironmentDimension(new Point(0, 0));
            System.err.println("PARSER ERROR: " + e.getMessage());
        }
    }
    
    public static void parseCreatureInfoFuelLevelMessage(String message) {
        // "Fuel: 0"
        try {
            String[] tokens = message.split(":");
            String sFuelLevel = tokens[tokens.length - 1].trim();
            int fuelLevel = Integer.parseInt(sFuelLevel);
            app.getModel().getCreature().setFuelLevel(fuelLevel);
        } catch (Exception e) {
            app.getModel().getWorld().setStarted(false);
            app.getModel().getWorld().setRunning(false);
            app.getModel().getWorld().setEnvironmentDimension(new Point(0, 0));
            System.err.println("PARSER ERROR: " + e.getMessage());
        }
    }
    
    public static void parseCreatureCoordsMessage(String message) {
        // "Fuel: 0"
        try {
            String[] tokens = message.split(":");
            tokens = tokens[1].split(" ");
            String sx = tokens[0].trim();
            String sy = tokens[1].trim();
            String sp = tokens[2].trim();
            int x = (int) Float.parseFloat(sx);
            int y = (int) Float.parseFloat(sy);
            int p = (int) Float.parseFloat(sp);
            app.getModel().getCreature().setLocation(new Point(x, y));
        } catch (Exception e) {
            app.getModel().getWorld().setStarted(false);
            app.getModel().getWorld().setRunning(false);
            app.getModel().getWorld().setEnvironmentDimension(new Point(0, 0));
            System.err.println("PARSER ERROR: " + e.getMessage());
        }
    }

    public static void parseCreatureStartedMessage(String message) {
        app.getModel().getCreature().setWorking(true);
    }
    
    public static void parseCreatureStoppedMessage(String message) {
        app.getModel().getCreature().setWorking(false);
    }
    
    public static String createNewCreatureMessage(int x, int y, int p) {
        String message = NEW_CREATURE_MESSAGE;
        message = message.replace("_x_", String.valueOf(x));
        message = message.replace("_y_", String.valueOf(y));
        message = message.replace("_p_", String.valueOf(p));
        return message;
    }

    public static String createCameraMessage(int creatureId) {
        String message = CAMERA_MESSAGE;
        message = message.replace("_id_", String.valueOf(creatureId));
        return message;
    }

    public static String createStartCreatureMessage(int creatureId) {
        String message = START_CREATURE_MESSAGE;
        message = message.replace("_id_", String.valueOf(creatureId));
        return message;
    }
    
    public static String createStopCreatureMessage(int creatureId) {
        String message = STOP_CREATURE_MESSAGE;
        message = message.replace("_id_", String.valueOf(creatureId));
        return message;
    }

    public static String createCreatureInfoMessage(int creatureId) {
        String message = CREATURE_INFO_MESSAGE;
        message = message.replace("_id_", String.valueOf(creatureId));
        return message;
    }

    public static String createCreatureCoordsMessage(int creatureId) {
        String message = CREATURE_COORDS_MESSAGE;
        message = message.replace("_id_", String.valueOf(creatureId));
        return message;
    }

    public static String createRefuelMessage(int creatureId) {
        String message = REFUEL_MESSAGE;
        message = message.replace("_id_", String.valueOf(creatureId));
        return message;
    }

    public static String createSetAngleMessage(int id, double velocity, double angle) {
        String message = SET_ANGLE_MESSAGE;
        DecimalFormat dc = new DecimalFormat("0.0");
        message = message.replace("_id_", String.valueOf(id));
        message = message.replace("_v_", dc.format(velocity).replace(",", "."));
        message = message.replace("_a_", dc.format(angle).replace(",", "."));
        return message;
    }

}
