package worldclient.model;

import java.awt.Point;
import java.util.Observable;
import java.util.logging.Level;
import java.util.logging.Logger;
import worldclient.app.WorldClientApp;

/**
 *
 * @author Fernando Martines
 */
public class Creature extends Observable {

    private static final double CREATURE_VELOCITY_STEP = 0.5;
    private static final double CREATURE_ANGLE_STEP    = 10;
    
    public static enum Property {
        CREATURE_ID_PROPERTY,
        CREATURE_FUEL_PROPERTY,
        CREATURE_CAMERA_PROPERTY,
        CREATURE_WORKING_PROPERTY,
        CREATURE_LOCATION_PROPERTY,
    }
    
    private int id;
    private int camera;
    private int fuelLevel;
    private double angle;
    private double velocity;
    private boolean working;
    private Point location;
    private CreatureThread creatureThread;

    public Creature() {
        id = -1;
        camera = -1;
        fuelLevel = 0;
        location = new Point(0, 0);
        
        angle = 0;
        velocity = CREATURE_VELOCITY_STEP;
    }

    public int getId() {
        return id;
    }

    public void setId(int id) {
        this.id = id;
        this.setChanged();
        this.notifyObservers(Property.CREATURE_ID_PROPERTY);
    }

    public int getCamera() {
        return camera;
    }

    public void setCamera(int camera) {
        this.camera = camera;
        this.setChanged();
        this.notifyObservers(Property.CREATURE_CAMERA_PROPERTY);
    }

    public Point getLocation() {
        return location;
    }

    public void setLocation(Point location) {
        this.location = location;
        this.setChanged();
        this.notifyObservers(Property.CREATURE_LOCATION_PROPERTY);
    }

    public void incAngle() {
        this.angle += CREATURE_ANGLE_STEP;
        if (this.angle > 360)
            this.angle = CREATURE_ANGLE_STEP;
    }

    public void decAngle() {
        this.angle -= CREATURE_ANGLE_STEP;
        if (this.angle < 0)
            this.angle = 360 - CREATURE_ANGLE_STEP;
    }

    public double getAngle() {
        return angle;
    }

    public void setAngle(double angle) {
        this.angle = angle;
    }

    public double getVelocity() {
        return velocity;
    }

    public void incVelocity() {
        this.velocity += CREATURE_VELOCITY_STEP;
        if (this.velocity > 10)
            this.velocity = 10;
    }

    public void decVelocity() {
        this.velocity -= CREATURE_VELOCITY_STEP;
        if (this.velocity <= -10)
            this.velocity = -10;
    }

    public int getFuelLevel() {
        return fuelLevel;
    }

    public void setFuelLevel(int fuelLevel) {
        this.fuelLevel = fuelLevel;
        this.setChanged();
        this.notifyObservers(Property.CREATURE_FUEL_PROPERTY);
    }

    public boolean isWorking() {
        return working;
    }

    public void setWorking(boolean working) {
        this.working = working;
        this.setChanged();
        this.notifyObservers(Property.CREATURE_WORKING_PROPERTY);
    }
   
    public void startObserver() {
        this.creatureThread = new CreatureThread();
        this.creatureThread.start();
    }
    
    private class CreatureThread extends Thread {
        
        @Override
        public void run() {            
            while (true) {
                try {
                    WorldClientApp.getInstance().getWorldClientController().syncCreatureInfo();
                    this.sleep(1000);
                    WorldClientApp.getInstance().getWorldClientController().syncCreatureCoords();
                    this.sleep(1000);
                } catch (InterruptedException ex) {
                    Logger.getLogger(Creature.class.getName()).log(Level.SEVERE, null, ex);
                }
            }            
        }        
    }
}
